/*
 *  dialogscroll.h  -  dialog scrolling when too high for screen
 *  Program:  kalarm
 *  Copyright © 2008 by David Jarvie <software@astrojar.org.uk>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#ifndef DIALOGSCROLL_H
#define DIALOGSCROLL_H

#include "kalarm.h"

#include <qscrollview.h>
#include <qvaluelist.h>
#include <qobjectlist.h>


/*=============================================================================
= Class DialogScroll
= A widget to contain the tab contents, allowing the contents to scroll if
= the dialog is too high to fit the screen.
=============================================================================*/
template <class T>
class DialogScroll : public QScrollView
{
	public:
		explicit DialogScroll(QWidget* parent = 0);
		~DialogScroll();
		QWidget*      widget() const    { return (QWidget*)viewport()->child(0, "QWidget", false); }
		virtual QSize sizeHint() const  { return minimumSizeHint(); }
		virtual QSize minimumSizeHint() const;
		static int    heightReduction() { return mHeightReduction; }
		static QSize  initMinimumHeight(T*);
		static void   setSized()        { mSized = true; }
		static bool   sized()           { return mSized; }
	private:
		static QValueList<DialogScroll<T>*> mTabs;
		static int    mMinHeight;
		static int    mHeightReduction;
		static bool   mSized;
};


#include "dialogscroll.h"
#include "functions.h"
#include <kdialog.h>
#include <qstyle.h>

template <class T> QValueList<DialogScroll<T>*> DialogScroll<T>::mTabs;
template <class T> int  DialogScroll<T>::mHeightReduction = 0;
template <class T> int  DialogScroll<T>::mMinHeight = -1;
template <class T> bool DialogScroll<T>::mSized = false;

template <class T>
DialogScroll<T>::DialogScroll(QWidget* parent)
	: QScrollView(parent)
{
	setFrameStyle(QFrame::NoFrame);
	setHScrollBarMode(QScrollView::AlwaysOff);
	setVScrollBarMode(QScrollView::Auto);
	setResizePolicy(QScrollView::AutoOneFit);
	setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
	mTabs += this;
}

template <class T>
DialogScroll<T>::~DialogScroll()
{
	mTabs.remove(this);
}

template <class T>
QSize DialogScroll<T>::minimumSizeHint() const
{
	QWidget* w = widget();
	if (!w)
		return QSize();
	int m = 2 * margin();
	QSize s = w->minimumSizeHint();
	s = QSize(s.width() + m, s.height() + m);
	if (mMinHeight > 0)
		return QSize(s.width() + style().pixelMetric(QStyle::PM_ScrollBarExtent), mMinHeight);
	return s;
}

/******************************************************************************
* Return the minimum size for the dialog.
* If the minimum size would be too high to fit the desktop, the tab contents
* are made scrollable.
*/
template <class T>
QSize DialogScroll<T>::initMinimumHeight(T* dlg)
{
	if (mSized)
		return QSize();
	int maxHeight = 0;
	for (typename QValueList<DialogScroll<T>*>::ConstIterator it = mTabs.constBegin();  it != mTabs.constEnd();  ++it)
	{
		QWidget* w = (*it)->widget();
		if (!w)
			return QSize();
		w->adjustSize();
		QSize ms = w->minimumSizeHint();
		QSize s = w->sizeHint();
		if (!s.isValid() || !ms.isValid() ||  !s.height() || !ms.height())
			return QSize();
		int h = QMAX(s.height(), ms.height());
		if (h > maxHeight)
			maxHeight = h;
		(*it)->setMinimumHeight(maxHeight);
	}
	int decoration = dlg->frameGeometry().height() - dlg->geometry().height();
	if (!decoration)
	{
		// On X11 at least, the window decoration height may not be
		// available, so use a guess of 25 pixels.
		decoration = 25;
	}
	int desk = KAlarm::desktopWorkArea().height();
	// There is no stored size, or the deferral group is visible.
	// Allow the tab contents to be scrolled vertically if that is necessary
	// to avoid the dialog exceeding the screen height.
	QSize s = dlg->KDialog::sizeHint();
	int y = s.height() + decoration - desk;
	if (y > 0)
	{
		mHeightReduction = y;
		mMinHeight = maxHeight - y;
		for (typename QValueList<DialogScroll<T>*>::ConstIterator it = mTabs.constBegin();  it != mTabs.constEnd();  ++it)
			(*it)->setMinimumHeight(mMinHeight);
		if (mMinHeight > 0)
		{
			for (typename QValueList<DialogScroll<T>*>::ConstIterator it = mTabs.constBegin();  it != mTabs.constEnd();  ++it)
			{
				DialogScroll<T>* ds = *it;
				ds->setMinimumHeight(mMinHeight);
				ds->resize(QSize(ds->width(), mMinHeight));
			}
		}
		mSized = true;
		for (QWidget* w = mTabs[0]->parentWidget();  w && w != dlg;  w = w->parentWidget())
		{
			w->adjustSize();
			w->setMinimumHeight(QMIN(w->minimumSizeHint().height(), w->sizeHint().height()));
			w->resize(w->minimumSize());
		}
		dlg->adjustSize();
		s = dlg->KDialogBase::minimumSizeHint();
		if (dlg->sizeHint().height() < s.height())
			s.setHeight(dlg->sizeHint().height());
		dlg->setMinimumHeight(s.height());
	}
	if (!mHeightReduction)
	{
		for (typename QValueList<DialogScroll<T>*>::ConstIterator it = mTabs.constBegin();  it != mTabs.constEnd();  ++it)
			(*it)->setMinimumHeight(maxHeight);
		mSized = true;
	}
	dlg->resize(s);
	return s;
}

#endif // DIALOGSCROLL_H
